@@ -1,45 +1,42 @@
 #[cfg(feature = "debugger")]
 use std::collections::HashSet;
-use std::env;
 use std::fs;
 use std::path::{Path, PathBuf};
+use std::process::Command;
 use std::process::Stdio;
-use std::process::{Command, Output};
 
-use colored::Colorize;
 use log::{debug, info, warn};
 use simpath::{FileType, FoundType, Simpath};
 use tempdir::TempDir;
 use url::Url;
 
-use flowcore::url_helper::url_from_string;
-
+use crate::compiler::cargo_build;
 use crate::errors::*;
 use crate::model::function::Function;
 
-/// Compile a function provided in rust to wasm and modify implementation to point to new file
-/// Checks the timestamp of source and wasm files and only recompiles if wasm file is out of date
+/// Compile a function's implementation to wasm and modify implementation to point to the wasm file
+/// Checks the timestamps of the source and wasm files and only recompiles if wasm file is out of date
 pub fn compile_implementation(
+    target_dir: &Path,
     function: &mut Function,
-    skip_building: bool,
+    native_only: bool,
     #[cfg(feature = "debugger")] source_urls: &mut HashSet<(Url, Url)>,
 ) -> Result<(PathBuf, bool)> {
     let mut built = false;
 
-    let (implementation_path, wasm_destination) = get_paths(function)?;
+    let (source_path, wasm_destination) = get_paths(target_dir, function)?;
 
     #[cfg(feature = "debugger")]
     source_urls.insert((
-        Url::from_file_path(&implementation_path)
-            .map_err(|_| "Could not create Url from file path")?,
+        Url::from_file_path(&source_path).map_err(|_| "Could not create Url from file path")?,
         Url::from_file_path(&wasm_destination)
             .map_err(|_| "Could not create Url from file path")?,
     ));
 
-    let (missing, out_of_date) = out_of_date(&implementation_path, &wasm_destination)?;
+    let (missing, out_of_date) = out_of_date(&source_path, &wasm_destination)?;
 
     if missing || out_of_date {
-        if skip_building {
+        if native_only {
             if missing {
                 let message = format!("Implementation at '{}' is missing and you have selected to skip building, so flows relaying on this implementation will not execute correctly.\nYou can build it using 'flowc', using the '-p' option", wasm_destination.display());
                 warn!("{}", message);
@@ -48,19 +45,27 @@ pub fn compile_implementation(
                 info!(
                     "Implementation at '{}' is out of date with source at '{}'",
                     wasm_destination.display(),
-                    implementation_path.display()
+                    source_path.display()
                 );
             }
         } else {
-            run_cargo_build(&implementation_path, &wasm_destination)?;
+            match function.build_type.as_str() {
+                "rust" => cargo_build::run(&source_path, &wasm_destination)?,
+                _ => bail!(
+                    "Unknown build type '{}' for function at '{}'",
+                    function.build_type,
+                    function.source_url
+                ),
+            }
 
+            optimize_wasm_file_size(&wasm_destination)?;
             built = true;
         }
     } else {
         debug!(
             "wasm at '{}' is up-to-date with source at '{}'",
             wasm_destination.display(),
-            implementation_path.display()
+            source_path.display()
         );
     }
 
@@ -83,7 +88,14 @@ fn run_optional_command(wasm_path: &Path, command: &str, mut args: Vec<String>)
     if let Ok(FoundType::File(command_path)) =
         Simpath::new("PATH").find_type(command, FileType::File)
     {
-        let tmp_dir = TempDir::new("wasm-opt")?;
+        // Create a temp directory for building in. Use `new_in` to make sure it is in the same FS as the destination so
+        // that fs::rename later works. It will be cleaned-up when `build_dir` goes out of scope.
+        let tmp_dir = TempDir::new_in(
+            wasm_path
+                .parent()
+                .ok_or("Could not get destination directory to create TempDir in")?,
+            "wasm-opt",
+        )?;
         let temp_file_path = tmp_dir
             .path()
             .join(wasm_path.file_name().ok_or("Could not get wasm filename")?);
@@ -106,7 +118,7 @@ fn run_optional_command(wasm_path: &Path, command: &str, mut args: Vec<String>)
             Some(0) | None => fs::rename(&temp_file_path, &wasm_path)?,
             Some(_) => bail!(format!(
                 "{} exited with non-zero status code",
-                command_path.to_string_lossy().to_string()
+                command_path.to_string_lossy()
             )),
         }
 
@@ -139,195 +151,23 @@ fn optimize_wasm_file_size(wasm_path: &Path) -> Result<()> {
     )
 }
 
-/*
-   Check the command Output for an error and print details if it failed
-*/
-fn check_cargo_error(command: &str, args: Vec<&str>, output: Output) -> Result<()> {
-    match output.status.code() {
-        Some(0) | None => Ok(()),
-        Some(code) => {
-            println!(
-                "{}\n{}",
-                "Process STDOUT:".green(),
-                String::from_utf8_lossy(&output.stdout).green()
-            );
-            println!(
-                "{}\n{}",
-                "Process STDERR:".red(),
-                String::from_utf8_lossy(&output.stderr).red()
-            );
-            bail!(
-                "cargo exited with status code: {}\nCommand Line: {} {:?}",
-                code,
-                command,
-                args
-            )
-        }
-    }
-}
-
-fn cargo_test(manifest_path: PathBuf, build_dir: PathBuf) -> Result<()> {
-    let command = "cargo";
-
-    debug!("Build directory: '{}'", build_dir.display());
-
-    let manifest_arg = format!("--manifest-path={}", manifest_path.display());
-    let target_dir_arg = format!("--target-dir={}", build_dir.display());
-    let test_args = vec!["test", "--quiet", &manifest_arg, &target_dir_arg];
-
-    println!(
-        "   {} {} WASM Project",
-        "Testing".green(),
-        manifest_path.display()
-    );
-
-    debug!("Running command = '{}', args = {:?}", command, test_args);
-
-    let output = Command::new(&command)
-        .args(&test_args)
-        .stdin(Stdio::inherit())
-        .stdout(Stdio::piped())
-        .stderr(Stdio::piped())
-        .output()
-        .chain_err(|| "Error while attempting to spawn cargo to test WASM Project")?;
-
-    check_cargo_error(command, test_args, output)
-}
-
-/*
-   Run the cargo command that builds the WASM output file
-*/
-fn cargo_build(
-    manifest_path: PathBuf,
-    build_dir: PathBuf,
-    implementation_path: &Path,
-    wasm_destination: &Path,
-) -> Result<()> {
-    let command = "cargo";
-    let manifest = format!("--manifest-path={}", manifest_path.display());
-    let target_dir = format!("--target-dir={}", build_dir.display());
-
-    println!(
-        "   {} {} WASM project",
-        "Compiling".green(),
-        manifest_path.display()
-    );
-
-    debug!(
-        "Building WASM '{}' from source '{}'",
-        wasm_destination.display(),
-        implementation_path.display()
-    );
-
-    let command_args = vec![
-        "build",
-        "--quiet",
-        "--release",
-        "--lib",
-        "--target=wasm32-unknown-unknown",
-        &manifest,
-        &target_dir,
-    ];
-
-    debug!(
-        "Running command = '{}', command_args = {:?}",
-        command, command_args
-    );
-
-    let output = Command::new(&command)
-        .env_remove("RUSTFLAGS") // remove flags for coverage, incompatible with wasm build
-        .args(&command_args)
-        .stdin(Stdio::inherit())
-        .stdout(Stdio::piped())
-        .stderr(Stdio::piped())
-        .output()
-        .chain_err(|| "Error while attempting to spawn cargo to compile WASM")?;
-
-    check_cargo_error(command, command_args, output)?;
-
-    // no error occurred, so move the built files to final destination and clean-up
-    let mut wasm_filename = implementation_path.to_path_buf();
-    wasm_filename.set_extension("wasm");
-    let mut wasm_build_location = build_dir.clone();
-    wasm_build_location.push("wasm32-unknown-unknown/release/");
-    wasm_build_location.push(
-        wasm_filename
-            .file_name()
-            .ok_or("Could not convert filename to str")?,
-    );
-    optimize_wasm_file_size(&wasm_build_location)?;
-
-    // copy compiled wasm output into place where flow's toml file expects it
-    fs::copy(&wasm_build_location, &wasm_destination)
-        .chain_err(|| "Could not copy WASM to destination")?;
-
-    // clean up temp dir
-    fs::remove_dir_all(&build_dir).chain_err(|| {
-        format!(
-            "Could not remove temporary build directory '{}'",
-            build_dir.display()
-        )
-    })
-}
-/*
-    Run the cargo build to compile wasm from function source
-*/
-fn run_cargo_build(implementation_path: &Path, wasm_destination: &Path) -> Result<()> {
-    let mut cargo_manifest_path = implementation_path.to_path_buf();
-    cargo_manifest_path.set_file_name("Cargo.toml");
-    let build_dir = TempDir::new("flow")
-        .chain_err(|| "Error creating new TempDir for compiling in")?
-        .into_path();
-
-    cargo_test(cargo_manifest_path.clone(), build_dir.clone())?;
-    cargo_build(
-        cargo_manifest_path,
-        build_dir,
-        implementation_path,
-        wasm_destination,
-    )
-}
-
 /*
    Calculate the paths to the source file of the implementation of the function to be compiled
-   and where to output the compiled wasm
+   and where to output the compiled wasm.
+
+   out_dir optionally overrides the destination directory where the wasm should end up
 */
-fn get_paths(function: &Function) -> Result<(PathBuf, PathBuf)> {
-    let cwd = env::current_dir().chain_err(|| "Could not get current working directory value")?;
-    let cwd_url = Url::from_directory_path(cwd)
-        .map_err(|_| "Could not form a Url for the current working directory")?;
+fn get_paths(target_dir: &Path, function: &Function) -> Result<(PathBuf, PathBuf)> {
+    let source_url = function.get_source_url().join(function.get_source())?;
 
-    let function_source_url = url_from_string(&cwd_url, Some(function.get_source_url()))
-        .chain_err(|| "Could not create a url from source url")?;
-    let implementation_source_url = function_source_url.join(function.get_implementation())?;
-
-    let implementation_source_path = implementation_source_url
+    let source_path = source_url
         .to_file_path()
         .map_err(|_| "Could not convert source url to file path")?;
-    if implementation_source_path
-        .extension()
-        .ok_or("No file extension on source file")?
-        .to_str()
-        .ok_or("Could not convert file extension to String")?
-        != "rs"
-    {
-        bail!(
-            "Source file at '{}' does not have a '.rs' extension",
-            implementation_source_path.display()
-        );
-    }
 
-    if !implementation_source_path.exists() {
-        bail!(
-            "Source file at '{}' does not exist",
-            implementation_source_path.display()
-        );
-    }
+    let mut wasm_path = target_dir.join(function.get_source());
+    wasm_path.set_extension("wasm");
 
-    let mut implementation_wasm_path = implementation_source_path.clone();
-    implementation_wasm_path.set_extension("wasm");
-
-    Ok((implementation_source_path, implementation_wasm_path))
+    Ok((source_path, wasm_path))
 }
 
 /*
@@ -358,10 +198,10 @@ fn out_of_date(source: &Path, derived: &Path) -> Result<(bool, bool)> {
 
 #[cfg(test)]
 mod test {
+    use std::{env, fs};
     #[cfg(feature = "debugger")]
-    use std::collections::HashSet;
-    use std::fs;
-    use std::fs::{remove_file, write, File};
+        use std::collections::HashSet;
+    use std::fs::{File, remove_file, write};
     use std::path::Path;
     use std::time::Duration;
 
@@ -378,8 +218,8 @@ mod test {
     use crate::model::io::IO;
     use crate::model::route::Route;
 
-    use super::out_of_date;
     use super::{get_paths, run_optional_command};
+    use super::out_of_date;
 
     #[test]
     fn test_run_optional_non_existent() {
@@ -488,16 +328,13 @@ mod test {
             "stdout.rs".to_string(),
             "print".into(),
             vec![],
-            vec![IO::new("String", Route::default())],
-            &format!(
-                "{}/{}",
-                Path::new(env!("CARGO_MANIFEST_DIR"))
-                    .parent()
-                    .expect("Error getting Manifest Dir")
-                    .display()
-                    .to_string(),
-                "flowc/tests/test-functions/stdio/stdout"
-            ),
+            vec![IO::new(vec!("String".into()), Route::default())],
+            Url::parse(&format!(
+                "file://{}/{}",
+                env!("CARGO_MANIFEST_DIR"),
+                "tests/test-functions/stdio/stdout"
+            ))
+            .expect("Could not create source Url"),
             Route::from("/flow0/stdout"),
             Some("tests/test-functions/stdio/stdout".to_string()),
             vec![OutputConnection::new(
@@ -520,8 +357,13 @@ mod test {
     fn paths_test() {
         let function = test_function();
 
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+        let expected_output_wasm = target_dir.join("stdout.wasm");
+
         let (impl_source_path, impl_wasm_path) =
-            get_paths(&function).expect("Error in 'get_paths'");
+            get_paths(&target_dir, &function).expect("Error in 'get_paths'");
 
         assert_eq!(
             format!(
@@ -537,20 +379,7 @@ mod test {
                 .to_str()
                 .expect("Error converting path to str")
         );
-        assert_eq!(
-            format!(
-                "{}/{}",
-                Path::new(env!("CARGO_MANIFEST_DIR"))
-                    .parent()
-                    .expect("Error getting Manifest Dir")
-                    .display()
-                    .to_string(),
-                "flowc/tests/test-functions/stdio/stdout.wasm"
-            ),
-            impl_wasm_path
-                .to_str()
-                .expect("Error converting path to str")
-        );
+        assert_eq!(expected_output_wasm, impl_wasm_path);
     }
 
     #[test]
@@ -561,7 +390,13 @@ mod test {
         #[cfg(feature = "debugger")]
         let mut source_urls = HashSet::<(Url, Url)>::new();
 
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+        let expected_output_wasm = target_dir.join("stdout.wasm");
+
         let (wasm_destination, built) = super::compile_implementation(
+            &target_dir,
             &mut function,
             true,
             #[cfg(feature = "debugger")]
@@ -570,13 +405,7 @@ mod test {
         .expect("compile_implementation() failed");
 
         assert!(!built);
-        assert_eq!(
-            Path::new(env!("CARGO_MANIFEST_DIR"))
-                .parent()
-                .expect("Error getting Manifest Dir")
-                .join("flowc/tests/test-functions/stdio/stdout.wasm"),
-            wasm_destination
-        );
+        assert_eq!(expected_output_wasm, wasm_destination);
     }
 
     #[test]
@@ -584,16 +413,17 @@ mod test {
     fn test_compile_implementation_skip_missing() {
         let mut function = test_function();
 
-        let expected_output_wasm = Path::new(env!("CARGO_MANIFEST_DIR"))
-            .parent()
-            .expect("Error getting Manifest Dir")
-            .join("flowc/tests/test-functions/stdio/stdout.wasm");
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+        let expected_output_wasm = target_dir.join("stdout.wasm");
         let _ = fs::remove_file(&expected_output_wasm);
 
         #[cfg(feature = "debugger")]
         let mut source_urls = HashSet::<(Url, Url)>::new();
 
         let (wasm_destination, built) = super::compile_implementation(
+            &target_dir,
             &mut function,
             true,
             #[cfg(feature = "debugger")]
@@ -609,17 +439,19 @@ mod test {
     #[serial(stdio_wasm_compile)]
     fn test_compile_implementation() {
         let mut function = test_function();
+        function.build_type = "rust".into();
 
-        let expected_output_wasm = Path::new(env!("CARGO_MANIFEST_DIR"))
-            .parent()
-            .expect("Error getting Manifest Dir")
-            .join("flowc/tests/test-functions/stdio/stdout.wasm");
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+        let expected_output_wasm = target_dir.join("stdout.wasm");
         let _ = fs::remove_file(&expected_output_wasm);
 
         #[cfg(feature = "debugger")]
         let mut source_urls = HashSet::<(Url, Url)>::new();
 
         let (wasm_destination, built) = super::compile_implementation(
+            &target_dir,
             &mut function,
             false,
             #[cfg(feature = "debugger")]
@@ -636,10 +468,11 @@ mod test {
     fn test_compile_implementation_not_needed() {
         let mut function = test_function();
 
-        let expected_output_wasm = Path::new(env!("CARGO_MANIFEST_DIR"))
-            .parent()
-            .expect("Error getting Manifest Dir")
-            .join("flowc/tests/test-functions/stdio/stdout.wasm");
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+        let expected_output_wasm = target_dir.join("stdout.wasm");
+
         let _ = fs::remove_file(&expected_output_wasm);
         write(&expected_output_wasm, b"file touched during testing")
             .expect("Could not write to file during testing");
@@ -647,6 +480,7 @@ mod test {
         let mut source_urls = HashSet::<(Url, Url)>::new();
 
         let (wasm_destination, built) = super::compile_implementation(
+            &target_dir,
             &mut function,
             false,
             #[cfg(feature = "debugger")]
@@ -662,12 +496,17 @@ mod test {
     #[serial(stdio_wasm_compile)]
     fn test_compile_implementation_invalid_paths() {
         let mut function = test_function();
-        function.implementation = "does_not_exist".into();
+        function.set_source("does_not_exist");
 
         #[cfg(feature = "debugger")]
         let mut source_urls = HashSet::<(Url, Url)>::new();
 
+        let target_dir = tempdir::TempDir::new("flow")
+            .expect("Could not create TempDir during testing")
+            .into_path();
+
         assert!(super::compile_implementation(
+            &target_dir,
             &mut function,
             true,
             #[cfg(feature = "debugger")]
