@@ -8,7 +8,7 @@ use log::info;
 use serde_derive::Serialize;
 use url::Url;
 
-use flowcore::flow_manifest::{FlowManifest, MetaData, DEFAULT_MANIFEST_FILENAME};
+use flowcore::flow_manifest::{DEFAULT_MANIFEST_FILENAME, FlowManifest, MetaData};
 use flowcore::function::Function as RuntimeFunction;
 use flowcore::input::Input;
 use flowcore::output_connection::Source;
@@ -222,8 +222,8 @@ mod test {
     use url::Url;
 
     use flowcore::input::InputInitializer;
-    use flowcore::output_connection::Source::Output;
     use flowcore::output_connection::{OutputConnection, Source};
+    use flowcore::output_connection::Source::Output;
 
     use crate::model::function::Function;
     use crate::model::io::IO;
@@ -237,16 +237,16 @@ mod test {
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![],
             vec![
-                IO::new("Value", Route::default()),
-                IO::new("String", Route::default()),
+                IO::new(vec!("Value".into()), Route::default()),
+                IO::new(vec!("String".into()), Route::default()),
             ],
-            "file:///fake/file",
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![
                 OutputConnection::new(
                     Source::default(),
@@ -278,7 +278,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'function_id': 1,
@@ -315,13 +315,13 @@ mod test {
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![],
-            vec![IO::new("String", Route::default())],
-            "file:///fake/file",
+            vec![IO::new(vec!("String".into()), Route::default())],
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![OutputConnection::new(
                 Source::default(),
                 1,
@@ -340,7 +340,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'function_id': 1,
@@ -369,13 +369,13 @@ mod test {
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![],
-            vec![IO::new("String", Route::default())],
-            "file:///fake/file",
+            vec![IO::new(vec!("String".into()), Route::default())],
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![OutputConnection::new(
                 Source::default(),
                 1,
@@ -394,7 +394,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'function_id': 1,
@@ -421,19 +421,19 @@ mod test {
 
     #[test]
     fn function_with_initialized_input_generation() {
-        let mut io = IO::new("String", Route::default());
+        let mut io = IO::new(vec!("String".into()), Route::default());
         io.set_initializer(&Some(InputInitializer::Once(json!(1))));
 
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![io],
             vec![],
-            "file:///fake/file",
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![],
             0,
             0,
@@ -442,7 +442,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'inputs': [
     {
       'initializer': {
@@ -467,19 +467,19 @@ mod test {
 
     #[test]
     fn function_with_constant_input_generation() {
-        let mut io = IO::new("String", Route::default());
+        let mut io = IO::new(vec!("String".into()), Route::default());
         io.set_initializer(&Some(InputInitializer::Always(json!(1))));
 
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![io],
             vec![],
-            "file:///fake/file",
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![],
             0,
             0,
@@ -488,7 +488,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'inputs': [
     {
       'initializer': {
@@ -513,18 +513,18 @@ mod test {
 
     #[test]
     fn function_with_array_input_generation() {
-        let io = IO::new("Array/String", Route::default());
+        let io = IO::new(vec!("Array/String".into()), Route::default());
 
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![io],
             vec![],
-            "file:///fake/file",
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![],
             0,
             0,
@@ -533,7 +533,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'inputs': [
     {}
   ]
@@ -556,13 +556,13 @@ mod test {
         Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![],
-            vec![IO::new("String", Route::default())],
-            "file:///fake/file",
+            vec![IO::new(vec!("String".into()), Route::default())],
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![OutputConnection::new(
                 Source::default(),
                 1,
@@ -589,7 +589,7 @@ mod test {
   'route': '/flow0/stdout',
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'function_id': 1,
@@ -602,7 +602,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'function_id': 1,
@@ -630,13 +630,13 @@ mod test {
         let function = Function::new(
             Name::from("Stdout"),
             false,
-            "lib://flowruntime/stdio/stdout".to_string(),
+            "lib://context/stdio/stdout".to_string(),
             Name::from("print"),
             vec![],
-            vec![IO::new("Array", Route::default())],
-            "file:///fake/file",
+            vec![IO::new(vec!("Array".into()), Route::default())],
+            Url::parse("file:///fake/file").expect("Could not parse Url"),
             Route::from("/flow0/stdout"),
-            Some("flowruntime/stdio/stdout".to_string()),
+            Some("context/stdio/stdout".to_string()),
             vec![OutputConnection::new(
                 Output("/0".into()),
                 1,
@@ -655,7 +655,7 @@ mod test {
         let expected = "{
   'id': 0,
   'flow_id': 0,
-  'implementation_location': 'lib://flowruntime/stdio/stdout/Stdout',
+  'implementation_location': 'lib://context/stdio/stdout/Stdout',
   'output_connections': [
     {
       'source': {
