@@ -3,7 +3,7 @@ use std::fmt;
 use std::mem::take;
 
 use error_chain::bail;
-use log::{debug, error};
+use log::{debug, error, trace};
 use serde_derive::{Deserialize, Serialize};
 use url::Url;
 
@@ -11,25 +11,25 @@ use flowcore::flow_manifest::MetaData;
 use flowcore::input::InputInitializer;
 
 use crate::compiler::loader::Validate;
-use crate::errors::Error;
 use crate::errors::*;
+use crate::errors::Error;
 use crate::model::connection::Connection;
 use crate::model::connection::Direction;
 use crate::model::connection::Direction::FROM;
 use crate::model::connection::Direction::TO;
+use crate::model::io::{IO, IOType};
 use crate::model::io::Find;
 use crate::model::io::IOSet;
-use crate::model::io::{IOType, IO};
 use crate::model::name::HasName;
 use crate::model::name::Name;
 use crate::model::process::Process;
 use crate::model::process::Process::FlowProcess;
 use crate::model::process::Process::FunctionProcess;
 use crate::model::process_reference::ProcessReference;
+use crate::model::route::{Route, RouteType};
 use crate::model::route::HasRoute;
 use crate::model::route::SetIORoutes;
 use crate::model::route::SetRoute;
-use crate::model::route::{Route, RouteType};
 
 /// `Flow` defines a parent or child flow in the nested flow hierarchy
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -53,6 +53,9 @@ pub struct Flow {
     /// `metadata` about flow author, versions etc
     #[serde(default)]
     pub metadata: MetaData,
+    /// Name of any docs file associated with this Flow
+    #[serde(default)]
+    pub(crate) docs: String,
 
     /// When the same process is used multiple times within a single flow, to disambiguate
     /// between them each one must be given an alias that is used to refer to it
@@ -125,19 +128,21 @@ impl fmt::Display for Flow {
 
 impl Default for Flow {
     fn default() -> Flow {
+        #[allow(clippy::unwrap_used)]
         Flow {
-            name: Name::default(),
-            id: 0,
-            alias: Name::default(),
-            source_url: Flow::default_url(),
-            route: Route::default(),
-            process_refs: vec![],
+            name: Default::default(),
             inputs: vec![],
             outputs: vec![],
+            process_refs: vec![],
             connections: vec![],
-            subprocesses: HashMap::new(),
-            lib_references: HashSet::new(),
-            metadata: MetaData::default(),
+            metadata: Default::default(),
+            docs: "".to_string(),
+            alias: Default::default(),
+            id: 0,
+            source_url: Url::parse("file://").unwrap(),
+            route: Default::default(),
+            subprocesses: Default::default(),
+            lib_references: Default::default(),
         }
     }
 }
@@ -176,7 +181,8 @@ impl SetRoute for Flow {
 }
 
 impl Flow {
-    fn default_url() -> Url {
+    /// Return a default value for a Url as part of a flow
+    pub fn default_url() -> Url {
         #[allow(clippy::unwrap_used)]
         Url::parse("file://").unwrap()
     }
@@ -190,6 +196,11 @@ impl Flow {
         }
     }
 
+    /// Get the name of any associated docs file
+    pub fn get_docs(&self) -> &str {
+        &self.docs
+    }
+
     /// Get a reference to the set of inputs this flow defines
     pub fn inputs(&self) -> &IOSet {
         &self.inputs
@@ -344,12 +355,12 @@ impl Flow {
     fn build_connection(&mut self, connection: &mut Connection, level: usize) -> Result<()> {
         match self.get_route_and_type(FROM, connection.from(), &None) {
             Ok(from_io) => {
-                debug!("Found connection source:\n{:#?}", from_io);
+                trace!("Found connection source:\n{:#?}", from_io);
                 // Iterate over all the destinations for this connection
                 for to_route in connection.to() {
                     match self.get_route_and_type(TO, to_route, from_io.get_initializer()) {
                         Ok(to_io) => {
-                            debug!("Found connection destination:\n{:#?}", to_io);
+                            trace!("Found connection destination:\n{:#?}", to_io);
                             let mut new_connection = connection.clone();
                             new_connection.connect(from_io.clone(), to_io, level)?;
                             self.connections.push(new_connection);
@@ -435,29 +446,30 @@ mod test {
             name: "test_flow".into(),
             alias: "test_flow".into(),
             inputs: vec![
-                IO::new_named("String", "string", "string"),
-                IO::new_named("Number", "number", "number"),
+                IO::new_named(vec!("String".into()), "string", "string"),
+                IO::new_named(vec!("Number".into()), "number", "number"),
             ],
             outputs: vec![
-                IO::new_named("String", "string", "string"),
-                IO::new_named("Number", "number", "number"),
+                IO::new_named(vec!("String".into()), "string", "string"),
+                IO::new_named(vec!("Number".into()), "number", "number"),
             ],
+            source_url: super::Flow::default_url(),
             ..Default::default()
         };
 
         let process_1 = Process::FunctionProcess(Function {
             name: "process_1".into(),
             id: 0,
-            inputs: vec![IO::new("String", "")],
-            outputs: vec![IO::new("String", "")],
+            inputs: vec![IO::new(vec!("String".into()), "")],
+            outputs: vec![IO::new(vec!("String".into()), "")],
             ..Default::default()
         });
 
         let process_2 = Process::FunctionProcess(Function {
             name: "process_2".into(),
             id: 1,
-            inputs: vec![IO::new("String", "")],
-            outputs: vec![IO::new("Number", "")],
+            inputs: vec![IO::new(vec!("String".into()), "")],
+            outputs: vec![IO::new(vec!("Number".into()), "")],
             ..Default::default()
         });
 
